@@ -88,6 +88,13 @@ def view(request, title=None):
     try:
         content = page.view_content()
     except hatta.error.NotFoundErr:
+        if request.wiki.fallback_url:
+            url = request.wiki.fallback_url
+            if '%s' in url:
+                url = url % werkzeug.url_quote(title)
+            else:
+                url = "%s/%s" % (url, werkzeug.url_quote(title))
+            return werkzeug.routing.redirect(url, code=303)
         if request.wiki.read_only:
             raise hatta.error.NotFoundErr(_(u"Page not found."))
 
@@ -157,6 +164,8 @@ def save(request, title):
                 return edit(request, title, preview=text.split('\n'),
                                  captcha_error=response.error_code)
         comment = request.form.get("comment", "")
+        if u'href="' in comment or u'http:' in comment:
+            raise hatta.error.ForbiddenErr()
         author = request.get_author()
         text = request.form.get("text")
         try:
@@ -172,8 +181,6 @@ def save(request, title):
                     if title == link:
                         raise hatta.error.ForbiddenErr(
                             _(u"This page is locked."))
-            if u'href="' in comment or u'http:' in comment:
-                raise hatta.error.ForbiddenErr()
             if text.strip() == '':
                 request.wiki.storage.delete_page(title, author, comment)
                 url = request.get_url(request.wiki.front_page)
@@ -182,9 +189,9 @@ def save(request, title):
                                        parent)
         else:
             text = u''
-            upload = request.files['data']
-            f = upload.stream
-            if f is not None and upload.filename is not None:
+            upload = request.files.get('data')
+            if upload and upload.stream and upload.filename:
+                f = upload.stream
                 request.wiki.storage.save_data(title, f.read(), author,
                                        comment, parent)
             else:
@@ -239,15 +246,14 @@ def atom(request):
                 'to_rev': rev,
             }, force_external=True)
         else:
-            url = request.adapter.build('revision', {
+            url = request.adapter.build('view', {
                 'title': title,
-                'rev': rev,
             }, force_external=True)
         feed.add(title, comment, content_type="text", author=author,
                  url=url, updated=date)
     rev = request.wiki.storage.repo_revision()
-    response = hatta.response.response(request, 'atom', feed.generate(), '/+feed',
-                             'application/xml', rev)
+    response = hatta.response.response(request, 'atom', feed.generate(),
+                                       '/+feed', 'application/xml', rev)
     response.make_conditional(request)
     return response
 
@@ -317,6 +323,8 @@ def render(request, title):
             temp_dir = tempfile.mkdtemp(dir=cache_dir)
             result_file = render(temp_dir)
             mercurial.util.rename(result_file, cache_file)
+        except hatta.error.UnsupportedMediaTypeErr:
+            return download(request, title)
         finally:
             rm_temp_dir(temp_dir)
     try:
@@ -537,6 +545,9 @@ def search(request):
 
     _ = request.wiki.gettext
 
+    def highlight_html(m):
+        return werkzeug.html.b(m.group(0), class_="highlight")
+
     def search_snippet(title, words):
         """Extract a snippet of text for search results."""
 
@@ -553,8 +564,7 @@ def search(request):
         min_pos = max(position - 60, 0)
         max_pos = min(position + 60, len(text))
         snippet = werkzeug.escape(text[min_pos:max_pos])
-        highlighted = werkzeug.html.b(match.group(0), class_="highlight")
-        html = regexp.sub(highlighted, snippet)
+        html = regexp.sub(highlight_html, snippet)
         return html
 
     def page_search(words, page, request):
